//мк ATtiny13a
//F_CPU = 9600000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>


//Обьявление типа unsigned char новым названием uchar (для дальнейшего использования в коде)
typedef unsigned char uchar;

#define TX PB4
#define SPEED 9600		//Скорость передачи
#define DISP 0          //Подстройка скорости; -30/+30


#define SEND_LOW  PORTB &= ~(1 << TX)		//Послать нуль
#define SEND_HIGH PORTB |= (1 << TX)		//Послать единицу

#define ENABLE_TIMER \
	TCCR0B &= 0b11111000 ;\
	TCCR0B |= 0b00000010;             
	
#define UNENABLE_TIMER \
	TCCR0B &= 0b11111000; \
	TCNT0 = 0;



uchar uart_buf = 0;
uchar bit_count = 0;

void send_byte(uchar byte);
static void send_bit(void);



//Обработчик прерывания таймера по совпадению
//ISR_BLOCK - блокирует глобальный флаг разрешения прерываний во время выполнения обработчика
ISR(TIM0_COMPA_vect, ISR_BLOCK) {
	send_bit();
}


int main(void) {
	DDRB = 0b00010000;
	DDRB |= (1 << TX);			//TX - выход (установка бита TX в регистре DDRB в 1)
	PORTB = 0xFF;

	//OSCCAL = 107; //???????????
	 
	TCCR0A = 0b00000010;		//Режим - сброс по совпадению (CTC)
	TCCR0B = 0b00000000;
	TIMSK0 = (1 << OCIE0A); 	//Разрешение прерывания по совпадению с OCR0A 

	OCR0A = (F_CPU/8) / SPEED - 1 + DISP;
   
	sei();
    while(1) {
     	_delay_ms(20);
	    send_byte(~PINB & 0x0F);
	}
}


void send_byte(uchar byte) {
	ENABLE_TIMER;
	SEND_LOW;			//Стартовый бит
	uart_buf = byte;	//Буфер для отсылки байта
	bit_count = 0;		//счетчик отправленных битов
}

void send_bit(void) {
	if(bit_count == 8) {
		UNENABLE_TIMER;
		SEND_HIGH;		//Стоповый бит
	}
	else {
		if(bit_is_set(uart_buf, 0))
			SEND_HIGH;
		else
			SEND_LOW;
		uart_buf >>= 1;
		bit_count++;
	}
}
